-- multilog stored procedures
CREATE OR REPLACE FUNCTION multilog.timestamp_from_millis(t bigint)
  RETURNS timestamp without time zone AS
$BODY$
DECLARE
  t_sec bigint;
  t_millis bigint;
BEGIN
  t_sec := t / 1000;
  t_millis := t % 1000;
  RETURN to_timestamp(t_sec) + t_millis * interval '1 ms';
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION multilog.timestamp_from_millis(bigint)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION multilog.syslog_message_count(t1 bigint, t2 bigint, exporters text, severities text, facilities text, message text) RETURNS TABLE(t timestamp, exp inet, sev int, cnt bigint) AS
$BODY$
DECLARE
  exporter_filter inet[];
  severity_filter int[];
  facility_filter int[];
  time1 timestamp;
  time2 timestamp;
  query text;
BEGIN
  IF (exporters IS NOT NULL) THEN
    exporter_filter := string_to_array(exporters,',');
  END IF;
  IF (severities IS NOT NULL) THEN
    severity_filter := string_to_array(severities,',');
  END IF;
  IF (facilities IS NOT NULL) THEN
    facility_filter := string_to_array(facilities,',');
  END IF;
  SELECT multilog.timestamp_from_millis(t1) INTO time1;
  SELECT multilog.timestamp_from_millis(t2) INTO time2;
  query := 'SELECT ''' || time2 || '''::timestamp, exporter, severity, count(1) AS cnt FROM multilog.syslog_message WHERE "time" >= $1 AND "time" < $2';
  IF (exporter_filter IS NOT NULL) THEN
	query := query || ' AND exporter = ANY($3)';
  END IF;
  IF (severity_filter IS NOT NULL) THEN 
    query := query || ' AND severity = ANY($4)';
  END IF;
  IF (facility_filter IS NOT NULL) THEN
    query := query || ' AND facility = ANY($5)';
  END IF;
  IF (message IS NOT NULL) THEN
    query := query || ' AND message LIKE $6';
  END IF;
  query := query || ' GROUP BY exporter, severity';
  RETURN QUERY EXECUTE query USING time1, time2, exporter_filter, severity_filter, facility_filter, message || '%';
END;
$BODY$
LANGUAGE plpgsql VOLATILE CALLED ON NULL INPUT COST 100;
ALTER FUNCTION multilog.syslog_message_count(bigint, bigint, text, text, text, text) OWNER TO postgres;

CREATE OR REPLACE FUNCTION multilog.syslog_mixed_count(t1 bigint, t2 bigint, sample bigint, exporters text, severities text) RETURNS TABLE(t timestamp, exp inet, sev int, cnt bigint) AS
$BODY$
DECLARE
  mod_t1 bigint;
  sample_t1 bigint;
  mod_t2 bigint;
  sample_t2 bigint;
  exporter_filter inet[];
  severity_filter int[];
  time1 timestamp;
  sample_time1 timestamp;
  sample_time2 timestamp;
  time2 timestamp;
  query text;
BEGIN
  mod_t1 := t1 % sample;
  sample_t1 := t1;
  IF (mod_t1 > 0) THEN
    sample_t1 := t1 + (sample - mod_t1);
  END IF;
  mod_t2 := t2 % sample;
  sample_t2 := t2 - mod_t2;
  IF (exporters IS NOT NULL) THEN
    exporter_filter := string_to_array(exporters,',');
  END IF;
  IF (severities IS NOT NULL) THEN
    severity_filter := string_to_array(severities,',');
  END IF;
  SELECT multilog.timestamp_from_millis(t1) INTO time1;
  SELECT multilog.timestamp_from_millis(t2) INTO time2;
  SELECT multilog.timestamp_from_millis(sample_t1) INTO sample_time1;
  SELECT multilog.timestamp_from_millis(sample_t2) INTO sample_time2;
  IF (sample_t1 <= t2) AND (sample_t2 >= t1) THEN
    query := 'SELECT ''' || time2 || '''::timestamp, exporter, severity, SUM(counted)::bigint FROM (SELECT exporter, severity, count(1) AS counted FROM multilog.syslog_message WHERE "time" >= $1 AND "time" < $2';
	IF (exporter_filter IS NOT NULL) THEN
	  query := query || ' AND exporter = ANY($7)';
        END IF;
	IF (severity_filter IS NOT NULL) THEN 
	  query := query || ' AND severity = ANY($8)';
	END IF;
    query := query || ' GROUP BY exporter, severity UNION ALL SELECT exporter, severity, SUM("count") AS counted FROM multilog.severity_exporter_counter WHERE "time" > $3 AND "time" <= $4';
	IF (exporter_filter IS NOT NULL) THEN
	  query := query || ' AND exporter = ANY($7)';
	END IF;
	IF (severity_filter IS NOT NULL) THEN 
	  query := query || ' AND severity = ANY($8)';
	END IF;
    query := query || ' GROUP BY exporter, severity UNION ALL SELECT exporter, severity, count(1) AS counted FROM multilog.syslog_message WHERE "time" >= $5 AND "time" < $6';
    IF (exporter_filter IS NOT NULL) THEN
      query := query || ' AND exporter = ANY($7)';
    END IF;
    IF (severity_filter IS NOT NULL) THEN 
      query := query || ' AND severity = ANY($8)';
    END IF;
    query := query || ' GROUP BY exporter, severity) AS total GROUP BY exporter, severity';
    RETURN QUERY EXECUTE query USING time1, sample_time1, sample_time1, sample_time2, sample_time2, time2, exporter_filter, severity_filter;
  ELSE
    query := 'SELECT ''' || time2 || '''::timestamp, exporter, severity, count(1) AS cnt FROM multilog.syslog_message WHERE "time" >= $1 AND "time" < $2';
	IF (exporter_filter IS NOT NULL) THEN
	  query := query || ' AND exporter = ANY($3)';
	END IF;
	IF (severity_filter IS NOT NULL) THEN 
	  query := query || ' AND severity = ANY($4)';
	END IF;
	query := query || ' GROUP BY exporter, severity';
    RETURN QUERY EXECUTE query USING time1, time2, exporter_filter, severity_filter;
  END IF;
END;
$BODY$
LANGUAGE plpgsql VOLATILE CALLED ON NULL INPUT COST 100;
ALTER FUNCTION multilog.syslog_mixed_count(bigint, bigint, bigint, text, text) OWNER TO postgres;

CREATE OR REPLACE FUNCTION multilog.syslog_statistic(t1 bigint, t2 bigint, sample bigint, exporters text, severities text, facilities text, message text) RETURNS TABLE(t timestamp, exp inet, sev int, cnt bigint) AS
$BODY$
DECLARE
  mod bigint;
  sample_t bigint;
  sample_start bigint;
  sample_end bigint;
  mixed_count boolean;
  tmp RECORD;
BEGIN
  IF (facilities IS NULL AND message IS NULL) THEN
    mixed_count := TRUE;
  ELSE 
    mixed_count := FALSE;
  END IF;
  mod := t1 % sample;
  sample_t := t1;
  IF (mod > 0) THEN
    sample_t := t1 + (sample - mod);
  END IF;
  sample_end := sample_t;
  sample_start := sample_t - sample;
  WHILE (sample_end <= t2) LOOP
    IF mixed_count THEN
      RETURN QUERY SELECT * FROM multilog.syslog_mixed_count(sample_start, sample_end, 300000, exporters, severities);
    ELSE 
      RETURN QUERY SELECT * FROM multilog.syslog_message_count(sample_start, sample_end, exporters, severities, facilities, message);
    END IF;
    sample_start := sample_end;
    sample_end := sample_end + sample;
  END LOOP;
  IF mixed_count THEN
    RETURN QUERY SELECT * FROM multilog.syslog_mixed_count(sample_start, sample_end, 300000, exporters, severities);
  ELSE
    RETURN QUERY SELECT * FROM multilog.syslog_message_count(sample_start, sample_end, exporters, severities, facilities, message);
  END IF;
END;
$BODY$
LANGUAGE plpgsql VOLATILE CALLED ON NULL INPUT COST 100;
ALTER FUNCTION multilog.syslog_statistic(bigint, bigint, bigint, text, text, text, text) OWNER TO postgres;

-- insert db_version
INSERT INTO framework.db_version(patch_number, patch_date) VALUES(4, current_timestamp);